@@ -780,6 +780,11 @@ class Test:
         p = regex.compile('(?iu)' + lower_char)
         self.expect(lambda: type(p.match(upper_char)), self.MATCH_CLASS)
 
+        self.expect(lambda: type(regex.match(r"(?i)a", "A")), self.MATCH_CLASS)
+        self.expect(lambda: type(regex.match(r"a(?i)", "A")), self.MATCH_CLASS)
+        self.expect(lambda: type(regex.match(r"(?in)a", "A")), self.MATCH_CLASS)
+        self.expect(lambda: regex.match(r"a(?in)", "A"), "None")
+
     def test_dollar_matches_twice(self):
         # $ matches the end of string, and just before the terminating \n.
         pattern = regex.compile('$')
@@ -897,10 +902,10 @@ class Test:
         self.expect(lambda: regex.findall(ur"(?u)\W+",
           u" \u0939\u093f\u0928\u094d\u0926\u0940,"),
           "[u' ', u',']")
-        self.expect(lambda: regex.split(ur"(?uz)\b",
+        self.expect(lambda: regex.split(ur"(?un)\b",
           u" \u0939\u093f\u0928\u094d\u0926\u0940,"),
           "[u' ', u'\\u0939\\u093f\\u0928\\u094d\\u0926\\u0940', u',']")
-        self.expect(lambda: regex.split(ur"(?uz)\B",
+        self.expect(lambda: regex.split(ur"(?un)\B",
           u" \u0939\u093f\u0928\u094d\u0926\u0940,"),
           "[u'', u' \\u0939', u'\\u093f', u'\\u0928', u'\\u094d', u'\\u0926', u'\\u0940,', u'']")
 
@@ -927,20 +932,20 @@ class Test:
 
         self.expect(lambda: regex.findall(r"^|\w+", "foo bar"),
           "['', 'foo', 'bar']")
-        self.expect(lambda: regex.findall(r"(?z)^|\w+", "foo bar"),
+        self.expect(lambda: regex.findall(r"(?n)^|\w+", "foo bar"),
           "['', 'foo', 'bar']")
         self.expect(lambda: regex.findall(r"(?r)^|\w+", "foo bar"),
           "['bar', 'foo', '']")
-        self.expect(lambda: regex.findall(r"(?rz)^|\w+", "foo bar"),
+        self.expect(lambda: regex.findall(r"(?nr)^|\w+", "foo bar"),
           "['bar', 'foo', '']")
 
         self.expect(lambda: [m[0] for m in regex.finditer(r"^|\w+", "foo bar")],
           "['', 'foo', 'bar']")
-        self.expect(lambda: [m[0] for m in regex.finditer(r"(?z)^|\w+",
+        self.expect(lambda: [m[0] for m in regex.finditer(r"(?n)^|\w+",
           "foo bar")], "['', 'foo', 'bar']")
         self.expect(lambda: [m[0] for m in regex.finditer(r"(?r)^|\w+",
           "foo bar")], "['bar', 'foo', '']")
-        self.expect(lambda: [m[0] for m in regex.finditer(r"(?rz)^|\w+",
+        self.expect(lambda: [m[0] for m in regex.finditer(r"(?nr)^|\w+",
           "foo bar")], "['bar', 'foo', '']")
 
         self.expect(lambda: regex.findall(r"\G\w{2}", "abcd ef"),
@@ -954,11 +959,11 @@ class Test:
 
         self.expect(lambda: regex.findall(r"q*", "qqwe"),
           "['qq', '', '', '']")
-        self.expect(lambda: regex.findall(r"(?z)q*", "qqwe"),
+        self.expect(lambda: regex.findall(r"(?n)q*", "qqwe"),
           "['qq', '', '', '']")
         self.expect(lambda: regex.findall(r"(?r)q*", "qqwe"),
           "['', '', 'qq', '']")
-        self.expect(lambda: regex.findall(r"(?rz)q*", "qqwe"),
+        self.expect(lambda: regex.findall(r"(?nr)q*", "qqwe"),
           "['', '', 'qq', '']")
 
         self.expect(lambda: regex.findall(".", "abcd", pos=1, endpos=3),
@@ -1050,7 +1055,7 @@ class Test:
     def test_zerowidth(self):
         # Issue 3262.
         self.expect(lambda: regex.split(r"\b", "a b"), "['a b']")
-        self.expect(lambda: regex.split(r"(?z)\b", "a b"),
+        self.expect(lambda: regex.split(r"(?n)\b", "a b"),
           "['', 'a', ' ', 'b', '']")
 
         # Issue 1647489.
@@ -1062,14 +1067,14 @@ class Test:
           "['bar', 'foo', '']")
         self.expect(lambda: [m[0] for m in
           regex.finditer(r"(?r)^|\w+", "foo bar")], "['bar', 'foo', '']")
-        self.expect(lambda: regex.findall(r"(?z)^|\w+", "foo bar"),
+        self.expect(lambda: regex.findall(r"(?n)^|\w+", "foo bar"),
           "['', 'foo', 'bar']")
         self.expect(lambda: [m[0] for m in
-          regex.finditer(r"(?z)^|\w+", "foo bar")], "['', 'foo', 'bar']")
-        self.expect(lambda: regex.findall(r"(?rz)^|\w+", "foo bar"),
+          regex.finditer(r"(?n)^|\w+", "foo bar")], "['', 'foo', 'bar']")
+        self.expect(lambda: regex.findall(r"(?nr)^|\w+", "foo bar"),
           "['bar', 'foo', '']")
         self.expect(lambda: [m[0] for m in
-          regex.finditer(r"(?rz)^|\w+", "foo bar")], "['bar', 'foo', '']")
+          regex.finditer(r"(?nr)^|\w+", "foo bar")], "['bar', 'foo', '']")
 
         self.expect(lambda: regex.split("", "xaxbxc"), "['xaxbxc']")
         self.expect(lambda: [m for m in regex.splititer("", "xaxbxc")],
@@ -1079,23 +1084,26 @@ class Test:
         self.expect(lambda: [m for m in regex.splititer("(?r)", "xaxbxc")],
           "['xaxbxc']")
 
-        self.expect(lambda: regex.split("(?z)", "xaxbxc"),
+        self.expect(lambda: regex.split("(?n)", "xaxbxc"),
           "['', 'x', 'a', 'x', 'b', 'x', 'c', '']")
-        self.expect(lambda: [m for m in regex.splititer("(?z)", "xaxbxc")],
+        self.expect(lambda: [m for m in regex.splititer("(?n)", "xaxbxc")],
           "['', 'x', 'a', 'x', 'b', 'x', 'c', '']")
 
-        self.expect(lambda: regex.split("(?rz)", "xaxbxc"),
+        self.expect(lambda: regex.split("(?nr)", "xaxbxc"),
           "['', 'c', 'x', 'b', 'x', 'a', 'x', '']")
-        self.expect(lambda: [m for m in regex.splititer("(?rz)", "xaxbxc")],
+        self.expect(lambda: [m for m in regex.splititer("(?nr)", "xaxbxc")],
           "['', 'c', 'x', 'b', 'x', 'a', 'x', '']")
 
     def test_scoped_and_inline_flags(self):
         # Issues 433028, #433024, #433027.
         self.expect(lambda: regex.search(r"(?i)Ab", "ab").span(), "(0, 2)")
         self.expect(lambda: regex.search(r"(?i:A)b", "ab").span(), "(0, 2)")
-        self.expect(lambda: regex.search(r"A(?i)b", "ab"), "None")
+        self.expect(lambda: regex.search(r"A(?i)b", "ab").span(), "(0, 2)")
+        self.expect(lambda: regex.search(r"A(?in)b", "ab"), "None")
 
-        self.expect(lambda: regex.search(r"(?-i)Ab", "ab", flags=regex.I),
+        self.expect(lambda: regex.search(r"(?-i)Ab", "ab",
+          flags=regex.I).span(), "(0, 2)")
+        self.expect(lambda: regex.search(r"(?n-i)Ab", "ab", flags=regex.I),
           "None")
         self.expect(lambda: regex.search(r"(?-i:A)b", "ab", flags=regex.I),
           "None")
@@ -1202,26 +1210,26 @@ class Test:
 
     def test_word_boundary(self):
         text = u'The quick ("brown") fox can\'t jump 32.3 feet, right?'
-        self.expect(lambda: regex.split(ur'(?z)\b', text), repr([u'', u'The',
+        self.expect(lambda: regex.split(ur'(?n)\b', text), repr([u'', u'The',
           u' ', u'quick', u' ("', u'brown', u'") ', u'fox', u' ', u'can', u"'",
           u't', u' ', u'jump', u' ', u'32', u'.', u'3', u' ', u'feet', u', ',
           u'right', u'?']))
-        self.expect(lambda: regex.split(ur'(?wz)\b', text), repr([u'', u'The',
+        self.expect(lambda: regex.split(ur'(?wn)\b', text), repr([u'', u'The',
           u' ', u'quick', u' ', u'(', u'"', u'brown', u'"', u')', u' ', u'fox',
           u' ', u"can't", u' ', u'jump', u' ', u'32.3', u' ', u'feet', u',',
           u' ', u'right', u'?', u'']))
 
         text = u"The  fox"
-        self.expect(lambda: regex.split(ur'(?z)\b', text), repr([u'', u'The',
+        self.expect(lambda: regex.split(ur'(?n)\b', text), repr([u'', u'The',
           u'  ', u'fox', u'']))
-        self.expect(lambda: regex.split(ur'(?wz)\b', text), repr([u'', u'The',
+        self.expect(lambda: regex.split(ur'(?wn)\b', text), repr([u'', u'The',
           u' ', u' ', u'fox', u'']))
 
         text = u"can't aujourd'hui l'objectif"
-        self.expect(lambda: regex.split(ur'(?z)\b', text), repr([u'', u'can',
+        self.expect(lambda: regex.split(ur'(?n)\b', text), repr([u'', u'can',
           u"'", u't', u' ', u'aujourd', u"'", u'hui', u' ', u'l', u"'",
           u'objectif', u'']))
-        self.expect(lambda: regex.split(ur'(?wz)\b', text), repr([u'',
+        self.expect(lambda: regex.split(ur'(?wn)\b', text), repr([u'',
           u"can't", u' ', u"aujourd'hui", u' ', u"l'", u'objectif', u'']))
 
     def test_branch_reset(self):
@@ -1313,6 +1321,16 @@ class Test:
         self.expect(lambda: regex.findall(ur"[^\P{Alpha}]", u"a0"), 
           repr([u"a"]))
 
+        self.expect(lambda: "".join(regex.findall(r"[^\d-h]", "a^b12c-h")),
+          "'a^bc'")
+        self.expect(lambda: "".join(regex.findall(r"[^\dh]", "a^b12c-h")),
+          "'a^bc-'")
+        self.expect(lambda: "".join(regex.findall(r"[^h\s\db]", "a^b 12c-h")),
+          "'a^c-'")
+        self.expect(lambda: "".join(regex.findall(r"[^b\w]", "a b")), "' '")
+        self.expect(lambda: "".join(regex.findall(r"[^b\S]", "a b")), "' '")
+        self.expect(lambda: "".join(regex.findall(r"[^8\d]", "a 1b2")), "'a b'")
+
     def test_various(self):
         tests = [
             # Test ?P< and ?P= extensions.
@@ -1385,9 +1403,11 @@ class Test:
             ('a.{4,5}b', 'acc\nccb', '', "None"),
             ('a.b', 'a\rb', '0', repr('a\rb')),
             # The new behaviour is that the inline flag affects only what follows.
-            ('a.b(?s)', 'a\nb', '', "None"),
+            ('a.b(?s)', 'a\nb', '0', repr('a\nb')),
+            ('a.b(?ns)', 'a\nb', '', "None"),
             ('(?s)a.b', 'a\nb', '0', repr('a\nb')),
-            ('a.*(?s)b', 'acc\nccb', '', "None"),
+            ('a.*(?s)b', 'acc\nccb', '0', repr('acc\nccb')),
+            ('a.*(?ns)b', 'acc\nccb', '', "None"),
             ('(?s)a.*b', 'acc\nccb', '0', repr('acc\nccb')),
             ('(?s)a.{4,5}b', 'acc\nccb', '0', repr('acc\nccb')),
 
@@ -1853,10 +1873,14 @@ class Test:
             # Check odd placement of embedded pattern modifiers.
 
             # Not an error under PCRE/PRE:
-            # New behaviour is that inline flags affect only what follows.
-            ('w(?i)', 'W', '0', "None"),
+            # When the new behaviour is turned on positional inline flags affect
+            # only what follows.
+            ('w(?i)', 'W', '0', "'W'"),
+            ('w(?in)', 'W', '0', "None"),
             ('w(?i)', 'w', '0', "'w'"),
+            ('w(?in)', 'w', '0', "'w'"),
             ('(?i)w', 'W', '0', "'W'"),
+            ('(?in)w', 'W', '0', "'W'"),
 
             # Comments using the x embedded pattern modifier.
 
@@ -1912,10 +1936,14 @@ xyzabc
             # Bug 114033: nothing to repeat.
             (r'(x?)?', 'x', '0', "'x'"),
             # Bug 115040: rescan if flags are modified inside pattern.
-            # New behaviour is that inline flags affect only what follows.
-            (r' (?x)foo ', 'foo', '0', "None"),
+            # If the new behaviour is turned on then positional inline flags
+            # affect only what follows.
+            (r' (?x)foo ', 'foo', '0', "'foo'"),
+            (r' (?nx)foo ', 'foo', '0', "None"),
             (r'(?x) foo ', 'foo', '0', "'foo'"),
+            (r'(?nx) foo ', 'foo', '0', "'foo'"),
             (r'(?x)foo ', 'foo', '0', "'foo'"),
+            (r'(?nx)foo ', 'foo', '0', "'foo'"),
             # Bug 115618: negative lookahead.
             (r'(?<!abc)(d.f)', 'abcdefdof', '0', "'dof'"),
             # Bug 116251: character class bug.
